#!/usr/bin/env python3
"""Test script for audio features in DryBox"""

import sys
import os
import wave
import struct
import time

# Add parent directory to path
sys.path.append(os.path.dirname(os.path.abspath(__file__)))

from UI.audio_player import AudioPlayer, PYAUDIO_AVAILABLE
from UI.audio_processor import AudioProcessor

def create_test_audio(filename="test_tone.wav", duration=2, frequency=440):
    """Create a test audio file with a sine wave"""
    sample_rate = 8000
    num_samples = int(sample_rate * duration)
    
    # Generate sine wave
    import math
    samples = []
    for i in range(num_samples):
        t = float(i) / sample_rate
        value = int(32767 * 0.5 * math.sin(2 * math.pi * frequency * t))
        samples.append(value)
    
    # Save to WAV file
    with wave.open(filename, 'wb') as wav_file:
        wav_file.setnchannels(1)
        wav_file.setsampwidth(2)
        wav_file.setframerate(sample_rate)
        wav_file.writeframes(struct.pack(f'{len(samples)}h', *samples))
    
    print(f"Created test audio file: {filename}")
    return filename

def test_audio_player():
    """Test audio player functionality"""
    print("\n=== Testing Audio Player ===")
    
    player = AudioPlayer()
    player.set_debug_callback(print)
    
    if PYAUDIO_AVAILABLE:
        print("PyAudio is available - testing playback")
        
        # Test playback
        client_id = 0
        if player.start_playback(client_id):
            print(f"Started playback for client {client_id}")
            
            # Create and play test audio
            test_file = create_test_audio()
            with wave.open(test_file, 'rb') as wav:
                data = wav.readframes(wav.getnframes())
                
            # Add audio data
            chunk_size = 640  # 320 samples * 2 bytes
            for i in range(0, len(data), chunk_size):
                chunk = data[i:i+chunk_size]
                player.add_audio_data(client_id, chunk)
                time.sleep(0.04)  # 40ms per chunk
            
            time.sleep(0.5)  # Let playback finish
            player.stop_playback(client_id)
            print(f"Stopped playback for client {client_id}")
            
            # Clean up
            os.remove(test_file)
    else:
        print("PyAudio not available - skipping playback test")
    
    # Test recording (works without PyAudio)
    print("\n=== Testing Recording ===")
    client_id = 1
    player.start_recording(client_id)
    
    # Add some test data
    test_data = b'\x00\x01' * 320  # Simple test pattern
    for i in range(10):
        player.add_audio_data(client_id, test_data)
    
    save_path = player.stop_recording(client_id, "test_recording.wav")
    if save_path and os.path.exists(save_path):
        print(f"Recording saved successfully: {save_path}")
        os.remove(save_path)
    else:
        print("Recording failed")
    
    player.cleanup()
    print("Audio player test complete")

def test_audio_processor():
    """Test audio processor functionality"""
    print("\n=== Testing Audio Processor ===")
    
    processor = AudioProcessor()
    processor.set_debug_callback(print)
    
    # Create test audio
    test_file = create_test_audio("test_input.wav", duration=1, frequency=1000)
    
    # Read test audio
    with wave.open(test_file, 'rb') as wav:
        test_data = wav.readframes(wav.getnframes())
    
    # Test various processing functions
    print("\nTesting normalize:")
    normalized = processor.normalize_audio(test_data, target_db=-6)
    save_path = processor.save_processed_audio(normalized, test_file, "normalized")
    if save_path:
        print(f"Saved: {save_path}")
        os.remove(save_path)
    
    print("\nTesting gain:")
    gained = processor.apply_gain(test_data, gain_db=6)
    save_path = processor.save_processed_audio(gained, test_file, "gained")
    if save_path:
        print(f"Saved: {save_path}")
        os.remove(save_path)
    
    print("\nTesting filters:")
    filtered = processor.apply_low_pass_filter(test_data)
    save_path = processor.save_processed_audio(filtered, test_file, "lowpass")
    if save_path:
        print(f"Saved: {save_path}")
        os.remove(save_path)
    
    # Clean up
    os.remove(test_file)
    print("\nAudio processor test complete")

def main():
    """Run all tests"""
    print("DryBox Audio Features Test")
    print("==========================")
    
    if not PYAUDIO_AVAILABLE:
        print("\nNOTE: PyAudio not installed. Playback tests will be skipped.")
        print("To install: sudo dnf install python3-devel portaudio-devel && pip install pyaudio")
    
    test_audio_player()
    test_audio_processor()
    
    print("\nAll tests complete!")

if __name__ == "__main__":
    main()