#!/usr/bin/env python3
"""
Test the audio pipeline (Codec2 + FSK) independently
"""

import sys
import os
import wave
import struct
import numpy as np

# Add parent directory to path
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from voice_codec import Codec2Wrapper, FSKModem, Codec2Mode, Codec2Frame

def test_codec_only():
    """Test just the codec2 encode/decode"""
    print("\n1. Testing Codec2 only...")
    
    codec = Codec2Wrapper(mode=Codec2Mode.MODE_1200)
    
    # Read test audio
    with wave.open("wav/input.wav", 'rb') as wf:
        # Read 320 samples (40ms at 8kHz)
        frames = wf.readframes(320)
        if len(frames) < 640:  # 320 samples * 2 bytes
            print("Not enough audio data")
            return False
            
    # Convert to samples
    samples = struct.unpack(f'{len(frames)//2}h', frames)
    print(f"Input: {len(samples)} samples, first 10: {samples[:10]}")
    
    # Encode
    encoded = codec.encode(frames)
    if encoded:
        print(f"Encoded: {len(encoded.bits)} bytes")
        print(f"First 10 bytes: {encoded.bits[:10].hex()}")
    else:
        print("Encoding failed!")
        return False
    
    # Decode
    decoded = codec.decode(encoded)
    if decoded is not None:
        print(f"Decoded: type={type(decoded)}, len={len(decoded)}")
        if hasattr(decoded, '__getitem__'):
            print(f"First 10 samples: {list(decoded[:10])}")
        
        # Save decoded audio
        with wave.open("wav/test_codec_only.wav", 'wb') as out:
            out.setnchannels(1)
            out.setsampwidth(2)
            out.setframerate(8000)
            if hasattr(decoded, 'tobytes'):
                out.writeframes(decoded.tobytes())
            else:
                # Convert to bytes
                import array
                arr = array.array('h', decoded)
                out.writeframes(arr.tobytes())
        print("Saved decoded audio to wav/test_codec_only.wav")
        return True
    else:
        print("Decoding failed!")
        return False

def test_full_pipeline():
    """Test the full Codec2 + FSK pipeline"""
    print("\n2. Testing full pipeline (Codec2 + FSK)...")
    
    codec = Codec2Wrapper(mode=Codec2Mode.MODE_1200)
    modem = FSKModem()
    
    # Read test audio
    with wave.open("wav/input.wav", 'rb') as wf:
        frames = wf.readframes(320)
        if len(frames) < 640:
            print("Not enough audio data")
            return False
    
    # Encode with Codec2
    encoded = codec.encode(frames)
    if not encoded:
        print("Codec encoding failed!")
        return False
    print(f"Codec2 encoded: {len(encoded.bits)} bytes")
    
    # Modulate with FSK
    modulated = modem.modulate(encoded.bits)
    print(f"FSK modulated: {len(modulated)} float samples")
    
    # Demodulate
    demodulated, confidence = modem.demodulate(modulated)
    print(f"FSK demodulated: {len(demodulated)} bytes, confidence: {confidence:.2f}")
    
    if confidence < 0.5:
        print("Low confidence demodulation!")
        return False
    
    # Create frame for decoding
    frame = Codec2Frame(
        mode=Codec2Mode.MODE_1200,
        bits=demodulated,
        timestamp=0,
        frame_number=0
    )
    
    # Decode with Codec2
    decoded = codec.decode(frame)
    if decoded is not None:
        print(f"Decoded: type={type(decoded)}, len={len(decoded)}")
        
        # Save decoded audio
        with wave.open("wav/test_full_pipeline.wav", 'wb') as out:
            out.setnchannels(1)
            out.setsampwidth(2)
            out.setframerate(8000)
            if hasattr(decoded, 'tobytes'):
                out.writeframes(decoded.tobytes())
            else:
                # Convert to bytes
                import array
                arr = array.array('h', decoded)
                out.writeframes(arr.tobytes())
        print("Saved decoded audio to wav/test_full_pipeline.wav")
        return True
    else:
        print("Codec decoding failed!")
        return False

def test_byte_conversion():
    """Test the byte conversion that happens in the protocol"""
    print("\n3. Testing byte conversion...")
    
    # Create test PCM data
    test_samples = [100, -100, 200, -200, 300, -300, 0, 0, 1000, -1000]
    
    # Method 1: array.tobytes()
    import array
    arr = array.array('h', test_samples)
    bytes1 = arr.tobytes()
    print(f"array.tobytes(): {len(bytes1)} bytes, hex: {bytes1.hex()}")
    
    # Method 2: struct.pack
    bytes2 = struct.pack(f'{len(test_samples)}h', *test_samples)
    print(f"struct.pack(): {len(bytes2)} bytes, hex: {bytes2.hex()}")
    
    # They should be the same
    print(f"Bytes match: {bytes1 == bytes2}")
    
    # Test unpacking
    unpacked = struct.unpack(f'{len(bytes1)//2}h', bytes1)
    print(f"Unpacked: {unpacked}")
    print(f"Matches original: {list(unpacked) == test_samples}")
    
    return True

def main():
    print("Audio Pipeline Test")
    print("=" * 50)
    
    # Change to DryBox directory if needed
    if os.path.basename(os.getcwd()) != 'DryBox':
        if os.path.exists('DryBox'):
            os.chdir('DryBox')
    
    # Ensure wav directory exists
    os.makedirs("wav", exist_ok=True)
    
    # Run tests
    codec_ok = test_codec_only()
    pipeline_ok = test_full_pipeline()
    bytes_ok = test_byte_conversion()
    
    print("\n" + "=" * 50)
    print("Test Results:")
    print(f"  Codec2 only: {'✅ PASS' if codec_ok else '❌ FAIL'}")
    print(f"  Full pipeline: {'✅ PASS' if pipeline_ok else '❌ FAIL'}")
    print(f"  Byte conversion: {'✅ PASS' if bytes_ok else '❌ FAIL'}")
    
    if codec_ok and pipeline_ok and bytes_ok:
        print("\n✅ All tests passed!")
        print("\nIf playback still doesn't work, check:")
        print("1. Is the audio data actually being sent? (check debug logs)")
        print("2. Is PyAudio stream format correct? (16-bit, 8kHz, mono)")
        print("3. Is the volume turned up?")
    else:
        print("\n❌ Some tests failed - this explains the playback issue")

if __name__ == "__main__":
    main()