#!/usr/bin/env python3
"""
Test script to verify audio setup for DryBox
"""

import os
import sys
import wave

def check_audio_file():
    """Check if input.wav exists and has correct format"""
    wav_path = "wav/input.wav"
    
    if not os.path.exists(wav_path):
        print(f"❌ {wav_path} not found!")
        return False
    
    try:
        with wave.open(wav_path, 'rb') as wf:
            channels = wf.getnchannels()
            framerate = wf.getframerate()
            sampwidth = wf.getsampwidth()
            nframes = wf.getnframes()
            duration = nframes / framerate
            
            print(f"✅ Audio file: {wav_path}")
            print(f"   Channels: {channels} {'✅' if channels == 1 else '❌ (should be 1)'}")
            print(f"   Sample rate: {framerate}Hz {'✅' if framerate == 8000 else '❌ (should be 8000)'}")
            print(f"   Sample width: {sampwidth * 8} bits {'✅' if sampwidth == 2 else '❌'}")
            print(f"   Duration: {duration:.2f} seconds")
            print(f"   Size: {os.path.getsize(wav_path) / 1024:.1f} KB")
            
            return channels == 1 and framerate == 8000
            
    except Exception as e:
        print(f"❌ Error reading {wav_path}: {e}")
        return False

def check_pyaudio():
    """Check if PyAudio is installed and working"""
    try:
        import pyaudio
        p = pyaudio.PyAudio()
        
        # Check for output devices
        output_devices = 0
        for i in range(p.get_device_count()):
            info = p.get_device_info_by_index(i)
            if info['maxOutputChannels'] > 0:
                output_devices += 1
        
        p.terminate()
        
        print(f"✅ PyAudio installed")
        print(f"   Output devices available: {output_devices}")
        return True
        
    except ImportError:
        print("❌ PyAudio not installed")
        print("   To enable playback, run:")
        print("   sudo dnf install python3-devel portaudio-devel")
        print("   pip install pyaudio")
        return False
    except Exception as e:
        print(f"❌ PyAudio error: {e}")
        return False

def check_dependencies():
    """Check all required dependencies"""
    deps = {
        'PyQt5': 'PyQt5',
        'numpy': 'numpy',
        'struct': None,  # Built-in
        'wave': None,    # Built-in
    }
    
    print("\nDependency check:")
    all_good = True
    
    for module_name, pip_name in deps.items():
        try:
            __import__(module_name)
            print(f"✅ {module_name}")
        except ImportError:
            print(f"❌ {module_name} not found")
            if pip_name:
                print(f"   Install with: pip install {pip_name}")
            all_good = False
    
    return all_good

def main():
    print("DryBox Audio Setup Test")
    print("=" * 40)
    
    # Change to DryBox directory if needed
    if os.path.basename(os.getcwd()) != 'DryBox':
        if os.path.exists('DryBox'):
            os.chdir('DryBox')
            print(f"Changed to DryBox directory: {os.getcwd()}")
    
    print("\nChecking audio file...")
    audio_ok = check_audio_file()
    
    print("\nChecking PyAudio...")
    pyaudio_ok = check_pyaudio()
    
    print("\nChecking dependencies...")
    deps_ok = check_dependencies()
    
    print("\n" + "=" * 40)
    if audio_ok and deps_ok:
        print("✅ Audio setup is ready!")
        if not pyaudio_ok:
            print("⚠️  Playback disabled (PyAudio not available)")
            print("    Recording will still work")
    else:
        print("❌ Audio setup needs attention")
        
    print("\nUsage tips:")
    print("1. Run the UI: python UI/main.py")
    print("2. Click 'Run Automatic Test' or press Space")
    print("3. Enable playback on Phone 2 with Ctrl+2")
    print("4. You'll hear the decoded audio after handshake completes")

if __name__ == "__main__":
    main()